import "../stylesheets/RegisterAndLogin.css";
import {FormEvent, useState} from "react";
import {setCookie} from "../jwt/Cookies.ts";
import { useNavigate } from 'react-router-dom';
import { jwtDecode } from 'jwt-decode';
import Modal from 'react-modal';


function Register() {
    const navigate = useNavigate();

    const [firstname, setFirstname] = useState('');
    const [lastname, setLastname] = useState('');
    const [email, setEmail] = useState('');
    const [password, setPassword] = useState('');

    const [errorModalIsOpen, setErrorModalIsOpen] = useState(false);
    const [errorMessage, setErrorMessage] = useState('');

    const isValidEmail = (email: string): boolean => {
        const emailRegex = /^[^\s@]+@[^\s@]+\.[^\s@]+$/;
        return emailRegex.test(email);
    };

    const isValidName = (name: string): boolean => {
        return name.trim() !== ''; // Prüfe, ob der Name nicht leer ist
    };

    const isValidPassword = (password: string): boolean => {
        return password.length >= 4; // Prüfe, ob das Passwort mindestens 4 Zeichen lang ist
    };

    const openErrorModal = (message: string) => {
        setErrorMessage(message);
        setErrorModalIsOpen(true);
    };

    const closeErrorModal = () => {
        setErrorModalIsOpen(false);
        setErrorMessage('');
    };

    const handleSubmit = async (e: FormEvent<HTMLFormElement>) => {
        e.preventDefault(); //verhindert, dass die Seite neu geladen wird, wenn das Formular abgeschickt wird

        if (!isValidEmail(email)) {
            openErrorModal('Ungültige E-Mail-Adresse. Frontend');
            return;
        }

        if (!isValidName(firstname) || !isValidName(lastname)) {
            openErrorModal('Vorname und Nachname dürfen nicht leer sein. Frontend');
            return;
        }

        if (!isValidPassword(password)) {
            openErrorModal('Das Passwort muss mindestens 4 Zeichen lang sein. Frontend');
            return;
        }
        await handleRegister();
    };

    const handleRegister = async () => {
        console.log("handle register")
        try {
            const requestBody = {
                firstname: firstname,
                lastname: lastname,
                email: email,
                password: password
            };
            console.log(requestBody)
            const res = await fetch("http://localhost:8080/api/v1/auth/register", {
                    method: "POST",
                    headers: {
                        "Content-Type": "application/json"
                    },
                    body: JSON.stringify(requestBody)
                }
            )
            if (res.ok) {
                const {token} = await res.json();
                console.log("Jwt Token not decoded" + token);
                const decodedToken = jwtDecode(token);
                // @ts-ignore
                const expirationDate = new Date(decodedToken.exp * 1000); //*1000 Anzahl der Sekunden seit 1.1.1970 darstellt - JS arbeitet jedoch normalerweise mit Millisekunden
                setCookie("Jwt", token, expirationDate);
                console.log("Jwt decoded " + decodedToken);
                navigate('/');
            } else {
                const errorBody = await res.json();
                if (res.status === 400 && errorBody.error === 'InvalidEmailException') {
                    window.alert("Fehler bei der Anmeldung: Ungültige E-Mail-Adresse.");
                    console.error('Fehler bei der Anmeldung: Ungültige E-Mail-Adresse.');
                } else if (res.status === 409 && errorBody.error === 'DataIntegrityViolationException') {
                    window.alert("Fehler bei der Anmeldung: E-Mail-Adresse bereits vorhanden.");
                    console.error('Fehler bei der Anmeldung: E-Mail-Adresse bereits vorhanden.');

                } else {
                    window.alert(`Fehler bei der Anmeldung: ${res.status} - ${res.statusText}`);
                    console.error('Fehler bei der Anmeldung:', res.status, res.statusText);
                }
            }

        } catch (e) {
        }
    };

    return (
        <main>
            <div className="register">
                <h2 className="h2">REGISTRIERUNG</h2>
                <form onSubmit={handleSubmit}>
                    <div className="form-group">
                        <input className="input"
                            type="text"
                            value={firstname}
                            onChange={(e) => setFirstname(e.target.value)}
                            placeholder="Name"
                        />
                    </div>
                    <div className="form-group">
                        <input className="input"
                            type="text"
                            value={lastname}
                            onChange={(e) => setLastname(e.target.value)}
                            placeholder="Nachname"
                        />
                    </div>
                    <div className="form-group">
                            <input className="input"
                                type="text"
                                value={email}
                                onChange={(e) => setEmail(e.target.value)}
                                placeholder="Mailadresse"
                            />
                    </div>
                    <div className="form-group">
                        <input className="input"
                            type="password"
                            value={password}
                            onChange={(e) => setPassword(e.target.value)}
                            placeholder="Passwort (mindestens 4 Zeichen)"
                        />
                    </div>
                    <div>
                            <button className="submitButton" type="submit">Anmelden</button>
                    </div>
                </form>
            </div>
            <Modal
                isOpen={errorModalIsOpen}
                onRequestClose={closeErrorModal}
                contentLabel="Fehlermeldung"
                className="errorField"
            >
                <div className="error-message">
                    <p>{errorMessage}</p>
                    <button onClick={closeErrorModal}>Schließen</button>
                </div>
            </Modal>
        </main>
    );
}

export default Register;